#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import os
from re import search

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Form abs path of a distro lists dir

def ListsDir(home_dir, is_src_distro_list):

	distro_lists_subdir = \
		f'distro_lists_{"src" if is_src_distro_list else "bin"}'

	return os.path.join(home_dir, distro_lists_subdir)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Get packages of a distro list (home_dir/branch/distro)

def DistroPackages(home_dir, is_src_distro_list, branch, distro):

	# Check existence of a distro list
	def CheckList(path, err=''):
		if not os.path.isfile(path):
			err += f'{"; " if err else ""}There is no "{path}" distro list'
			return False, err
		return True, err
	# - nested func

	lists_dir = ListsDir(home_dir, is_src_distro_list)

	# Forming variants of paths for a distro list
	distro_list_path_variants = [
		os.path.join(lists_dir, branch, distro),
		os.path.join(lists_dir, distro), # Without branch specification
		]
	# With '.bnames' suffix
	distro_list_path_variants += \
		[f'{path}.bnames' for path in distro_list_path_variants]

	# Searching a distro list in the file system
	distro_list_path = ''
	err = ''
	for path in distro_list_path_variants:
		ok, err = CheckList(path, err)
		if ok:
			distro_list_path = path
			err = ''
			break
	if err:
		return set(), err

	packages = set()

	try:
		with open(distro_list_path, 'r') as f:
			for line in f.readlines():
				s = search(r'^[^\s]+', line.strip())
				if s:
					packages.add(s.group())
	except OSError:
		err = f'Can\'t read "{distro_list_path}" distro list file'
		return set(), err

	return packages, ''
